@@ -37,6 +37,7 @@
 #include "muse_astro.h"
 #include "muse_combine.h"
 #include "muse_pfits.h"
+#include "muse_phys.h"
 #include "muse_quality.h"
 #include "muse_utils.h"
 #include "muse_wcs.h"
@@ -59,173 +60,6 @@
 
 /**@{*/
 
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the saturation pressure using the Owens calibration.
-  @param   temp   temperature (in Kelvin)
-  @return  the saturation pressure for the given temperature
-
-  This function is currently also used when choosing the Filippenko formulae.
- */
-/*----------------------------------------------------------------------------*/
-static double
-muse_dar_owens_saturation_pressure(double temp)
-{
-  return -10474.0 + 116.43*temp - 0.43284*temp*temp + 0.00053840*pow(temp,3);
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the two coefficients for the Owens method.
-  @param   temp   temperature (in Kelvin)
-  @param   rhum   the relative humidity (in %)
-  @param   pres   atmospheric pressure (in hPa)
-  @param   d1     coefficient 1
-  @param   d2     coefficient 2
- */
-/*----------------------------------------------------------------------------*/
-static void
-muse_dar_owens_coeffs(double temp, double rhum, double pres,
-                      double *d1, double *d2)
-{
-  /* pressure helpers: saturation pressure */
-  double ps = muse_dar_owens_saturation_pressure(temp),
-         p2 = rhum * ps, /* water vapor pressure */
-         p1 = pres - p2; /* dry air pressure */
-  /* helpers/coefficients */
-  *d1 = p1/temp * (1. + p1*(57.90e-8 - 9.3250e-4/temp + 0.25844/temp/temp));
-  *d2 = p2/temp * (1. + p2*(1. + 3.7e-4*p2)
-                        * (-2.37321e-3 + 2.23366/temp - 710.792/temp/temp
-                           + 7.75141e4/pow(temp,3)));
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the refractive index for the given wavelength following
-           Owens.
-  @param   l    the wavelength (in um)
-  @param   d1   coefficient 1
-  @param   d2   coefficient 2
-  @return  the refractive index
- */
-/*----------------------------------------------------------------------------*/
-static inline double
-muse_dar_owens_nrindex(double l, double d1, double d2)
-{
-  double lisq = 1./(l*l); /* inverse square of the wavelength */
-  return ((2371.34 + 683939.7/(130. - lisq) + 4547.3/(38.9 - lisq)) * d1
-          + (6487.31 + 58.058 * lisq - 0.71150 * lisq*lisq
-             + 0.08851 * lisq*lisq*lisq) * d2) * 1.0e-8;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the refractive index for the given wavelength following
-           Filippenko.
-  @param   l    the wavelength (in um)
-  @param   T    temperature (in degrees Celsius)
-  @param   f    water vapor pressure (in mmHg)
-  @param   P    atmospheric pressure (in mmHg)
-  @return  the refractive index
- */
-/*----------------------------------------------------------------------------*/
-static inline double
-muse_dar_filippenko_nrindex(double l, double T, double f, double P)
-{
-  double lisq = 1./(l*l); /* inverse square of the wavelength */
-  /* 10^6 [n(lambda) - 1] at standard environmental conditions, Eq. (1) */
-  double nl = 64.328 + 29498.1 / (146 - lisq) + 255.4 / (41 - lisq);
-  /* correction for non-standard conditions, Eq. (2) */
-  nl *= P / 720.883 * (1 + (1.049 - 0.0157 * T) * 1e-6 * P) / (1 + 0.003661 * T);
-  /* correction for water vapor, Eq. (3) */
-  nl -= (0.0624 - 0.000680 * lisq) / (1 + 0.003661 * T) * f;
-  /* convert to refractive index n(lambda) */
-  nl = nl * 1e-6 + 1;
-  return nl;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the refractive index for the given wavelength following
-           Edlén.
-  @param   l      the wavelength [um]
-  @param   t      temperature [Celsius]
-  @param   p      pressure [Pa]
-  @param   pv     partial pressure [Pa]
-  @return  the refractive index
- */
-/*----------------------------------------------------------------------------*/
-static inline double
-muse_dar_edlen_nrindex(double l, double t, double p, double pv)
-{
-  /* define constants */
-  const double A = 8342.54, B = 2406147,
-               C = 15998, D = 96095.43,
-               E = 0.601, F = 0.00972, G = 0.003661;
-  double S = 1./(l*l), /* supposed to be "laser vacuum wavelength"?! XXX */
-         /* intermediate result for p, pv, and t */
-         n_5 = 1 + 1e-8 * (A + B / (130 - S) + C / (38.9 - S)),
-         X = (1 + 1e-8 * (E - F * t) * p) / (1 + G * t),
-         n_tp = 1 + p * (n_5 - 1) * X / D,
-         n = n_tp - 10e-10 * (292.75 / (t + 273.15)) * (3.7345 - 0.0401 * S) * pv;
-  return n;
-} /* muse_dar_edlen_nrindex() */
-
-/*----------------------------------------------------------------------------*/
-/**
-  @private
-  @brief   Compute the refractive index for the given wavelength following
-           Ciddor.
-  @param   l      the wavelength [um]
-  @param   T      temperature [K]
-  @param   p      pressure [Pa]
-  @param   xv     mole fraction
-  @param   xCO2   CO2 concentration [umol / mol], use 450.0 for a good default
-                  value
-  @return  the refractive index
-
-  The source of the value xCO2 = 450 is Birch et al., 1993, Metrologia 30, 7
-  (see the routine p3d_darc_amplitude_iag in the p3d IFS reduction package).
- */
-/*----------------------------------------------------------------------------*/
-static inline double
-muse_dar_ciddor_nrindex(double l, double T, double p, double xv, double xCO2)
-{
-  /* define constants */
-  const double w0 = 295.235, w1 = 2.6422, w2 = -0.03238, w3 = 0.004028,
-               k0 = 238.0185, k1 = 5792105, k2 = 57.362, k3 = 167917,
-               a0 = 1.58123e-6, a1 = -2.9331e-8, a2 = 1.1043e-10,
-               b0 = 5.707e-6, b1 = -2.051e-8,
-               c0 = 1.9898e-4, c1 = -2.376e-6,
-               d = 1.83e-11, e = -0.765e-8,
-               p_R1 = 101325, T_R1 = 288.15,
-               Z_a = 0.9995922115,
-               rho_vs = 0.00985938,
-               R = 8.314472, M_v = 0.018015;
-  double t = T - 273.15, /* temperature [Celsius] */
-         S = 1./(l*l), /* supposed to be "laser vacuum wavelength"?! XXX */
-         /* intermediate results that depend on S */
-         r_as = 1e-8 * ((k1 / (k0 - S)) + (k3 / (k2 - S))),
-         r_vs = 1.022e-8 * (w0 + w1 * S + w2 * S*S + w3 * S*S*S),
-         /* using the CO2 concentration, calculate intermediate values for CO2 */
-         M_a = 0.0289635 + 1.2011e-8 * (xCO2 - 400),
-         r_axs = r_as * (1 + 5.34e-7 * (xCO2 - 450)),
-         /* compressibility and density components */
-         Z_m = 1 - (p / T) * (a0 + a1 * t + a2 * t*t + (b0 + b1 * t) * xv
-                              + (c0 + c1 * t) * xv*xv)
-             + (p/T)*(p/T) * (d + e * xv*xv),
-         rho_axs = p_R1 * M_a / (Z_a * R * T_R1),
-         rho_v = xv * p * M_v / (Z_m * R * T),
-         rho_a = (1 - xv) * p * M_a / (Z_m * R * T),
-         n = 1 + (rho_a / rho_axs) * r_axs + (rho_v / rho_vs) * r_vs;
-  return n;
-} /* muse_dar_ciddor_nrindex() */
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief   Correct the pixel coordinates of all pixels of a given pixel table
@@ -268,16 +102,15 @@ muse_dar_ciddor_nrindex(double l, double T, double p, double xv, double xCO2)
     use it in a GPL program. The formulae originally come from J. C. Owens,
     1967 Applied Optics 6, 51 - 59.
   - "Edlen": the algorithm from B. Edlén, 1966 Metrologia 2, 71 - 80 and
-    K. P. Birch & M. J. Downs, 1993 Metrologia 30, 155 - 162. See
-    http://emtoolbox.nist.gov/Wavelength/Documentation.asp
-    for all formulae used.
+    K. P. Birch & M. J. Downs, 1993 Metrologia 30, 155 - 162.
   - "Ciddor": the algorithm from Phillip E. Ciddor, 1996 Applied Optics 35,
     1566 - 1573. This method is supposed to be the most accurate way to compute
-    the refractive index of air, with a validity range of at least 3000 to 17000
-    Angstrom, and was chosen by the International Association of Geodesy as
-    their standard (http://emtoolbox.nist.gov/Wavelength/Documentation.asp#EdlenorCiddor).
-    See http://emtoolbox.nist.gov/Wavelength/Documentation.asp#AppendixA for all
-    formulae used here.
+    the refractive index of air.
+  These algorithms differ in the way they compute the refractive index of air.
+  See http://emtoolbox.nist.gov/Wavelength/Documentation.asp#AppendixA for the
+  formulae used here and muse_phys_air_to_vacuum() as well as the other
+  functions in the muse_phys module for more information.
+
   Use the environment variable MUSE_DAR_CORRECT_METHOD to switch the default
   from Filippenko to something else. They differ in the way the refractive index
   of air is computed from the environmental parameters.
@@ -403,8 +236,8 @@ muse_dar_correct(muse_pixtable *aPixtable, double aLambdaRef)
          pv, /* partial vapor pressure */
          xv; /* mole fraction */
   if (method == MUSE_DAR_METHOD_OWENS) {
-    muse_dar_owens_coeffs(temp, rhum, pres, &d1, &d2);
-    nr0 = muse_dar_owens_nrindex(aLambdaRef / 10000., d1, d2);
+    muse_phys_nrindex_owens_coeffs(temp, rhum, pres, &d1, &d2);
+    nr0 = muse_phys_nrindex_owens(aLambdaRef / 10000., d1, d2);
     cpl_msg_info(__func__, "DAR for T=%.2f K, RH=%.2f %%, pres=%.1f mbar, "
                  "at airmass=%.4f, using ref. wavelength %.6f um (Owens)",
                  temp, rhum*100., pres, 1./cos(z), aLambdaRef / 10000.);
@@ -458,24 +291,22 @@ muse_dar_correct(muse_pixtable *aPixtable, double aLambdaRef)
       xv = rhum * f * psv_i / p; /* use saturation pressure over ice */
     }
     if (method == MUSE_DAR_METHOD_EDLEN) {
-      nr0 = muse_dar_edlen_nrindex(aLambdaRef / 10000., t, p, pv);
+      nr0 = muse_phys_nrindex_edlen(aLambdaRef / 10000., t, p, pv);
     } else { /* MUSE_DAR_METHOD_CIDDOR */
-      nr0 = muse_dar_ciddor_nrindex(aLambdaRef / 10000., T, p, xv, 450.);
+      nr0 = muse_phys_nrindex_ciddor(aLambdaRef / 10000., T, p, xv, 450.);
     }
     cpl_msg_info(__func__, "DAR for T=%.2f degC, RH=%.2f %%, p=%.1f Pa, "
                  "at airmass=%.4f, using ref. wavelength %.6f um (%s)",
                  t, rhum*100., p, 1./cos(z), aLambdaRef / 10000.,
                  method == MUSE_DAR_METHOD_EDLEN ? "Edlen" : "Ciddor");
   } else {
-#define CONVERT_hPa_TO_mmHg 0.75006158 /* conversion from hPa (or mbar) to *
-                                        * mmHg, needed for Filippenko      */
     /* use the Owens formula to derive saturation pressure, still needs T[K] */
-    double ps = muse_dar_owens_saturation_pressure(temp);
+    double ps = muse_phys_nrindex_owens_saturation_pressure(temp);
     /* using that, derive the water vapor pressure in mmHg */
-    fp = rhum * ps * CONVERT_hPa_TO_mmHg;
+    fp = rhum * ps * MUSE_PHYS_hPa_TO_mmHg;
     temp -= 273.15; /* temperature (again) in degrees Celsius */
-    pres *= CONVERT_hPa_TO_mmHg; /* need the pressure in mmHg */
-    nr0 = muse_dar_filippenko_nrindex(aLambdaRef / 10000., temp, fp, pres);
+    pres *= MUSE_PHYS_hPa_TO_mmHg; /* need the pressure in mmHg */
+    nr0 = muse_phys_nrindex_filippenko(aLambdaRef / 10000., temp, fp, pres);
     cpl_msg_info(__func__, "DAR for T=%.2f degC, fp=%.3f mmHg, P=%.1f mmHg, "
                  "at airmass=%.4f, using ref. wavelength %.6f um (Filippenko)",
                  temp, fp, pres, 1./cos(z), aLambdaRef / 10000.);
@@ -520,13 +351,13 @@ muse_dar_correct(muse_pixtable *aPixtable, double aLambdaRef)
   for (wl = 4650.; wl <= 9300; wl += 50) {
     double nr;
     if (method == MUSE_DAR_METHOD_OWENS) {
-      nr = muse_dar_owens_nrindex(wl / 10000., d1, d2);
+      nr = muse_phys_nrindex_owens(wl / 10000., d1, d2);
     } else if (method == MUSE_DAR_METHOD_EDLEN) {
-      nr = muse_dar_edlen_nrindex(wl / 10000., t, p, pv);
+      nr = muse_phys_nrindex_edlen(wl / 10000., t, p, pv);
     } else if (method == MUSE_DAR_METHOD_CIDDOR) {
-      nr = muse_dar_ciddor_nrindex(wl / 10000., temp, p, xv, 450.);
+      nr = muse_phys_nrindex_ciddor(wl / 10000., temp, p, xv, 450.);
     } else {
-      nr = muse_dar_filippenko_nrindex(wl / 10000., temp, fp, pres);
+      nr = muse_phys_nrindex_filippenko(wl / 10000., temp, fp, pres);
     }
     double dr = dr0 * (nr0 - nr);
     printf("%.1f Angstrom: %f ==> %f %f %s (%s)\n", wl, dr,
@@ -545,13 +376,13 @@ muse_dar_correct(muse_pixtable *aPixtable, double aLambdaRef)
   for (i = 0; i < nmax; i++) {
     double nr, lambda = lbda[i] * 1e-4;
     if (method == MUSE_DAR_METHOD_OWENS) {
-      nr = muse_dar_owens_nrindex(lambda, d1, d2);
+      nr = muse_phys_nrindex_owens(lambda, d1, d2);
     } else if (method == MUSE_DAR_METHOD_EDLEN) {
-      nr = muse_dar_edlen_nrindex(lambda, t, p, pv);
+      nr = muse_phys_nrindex_edlen(lambda, t, p, pv);
     } else if (method == MUSE_DAR_METHOD_CIDDOR) {
-      nr = muse_dar_ciddor_nrindex(lambda, temp, p, xv, 450.);
+      nr = muse_phys_nrindex_ciddor(lambda, temp, p, xv, 450.);
     } else {
-      nr = muse_dar_filippenko_nrindex(lambda, temp, fp, pres);
+      nr = muse_phys_nrindex_filippenko(lambda, temp, fp, pres);
     }
     double dr = dr0 * (nr0 - nr);
     /* correct coordinates directly in the pixel table */
@@ -649,17 +480,26 @@ muse_dar_check(muse_pixtable *aPixtable, double *aMaxShift,
   if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_DAR_CHECK)) {
     cpl_msg_info(__func__, "pixel table already checked for DAR accuracy");
   }
+  cpl_boolean corrected = CPL_FALSE;
   if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_DAR_CORR)) {
     cpl_msg_info(__func__, "pixel table already corrected for DAR residuals");
+    corrected = CPL_TRUE;
   }
   /* set the halfsize of the window to measure the centroids with respect to *
    * the reference wavelength; +/-5 pix should suffice in case DAR was       *
    * previously corrected; in other cases, increase it to +/-15 pix          */
   int cenhalfsize = 5;
-  /* polynomial order for the fits, default 2, if not corrected yet, use 4 */
-  int order = 2;
+  /* polynomial order for the fits, default 2, if not corrected yet, use 4; *
+   * here use the existence of the header keyword only as a first guess     */
+  cpl_boolean residuals = cpl_propertylist_has(aPixtable->header,
+                                               MUSE_HDR_PT_DAR_NAME);
+  int order = residuals || corrected ? 2 : 4;
 
   /* create cube, with coarse sampling in wavelength */
+  cpl_msg_info(__func__, "Intermediate resampling to %s atmospheric refraction"
+               "%s (using polynomial order %d):", aCorrect ? "correct" : "check",
+               residuals ? " residuals" : "", order);
+  cpl_msg_indent_more();
   muse_resampling_params *params =
     muse_resampling_params_new(MUSE_RESAMPLE_WEIGHTED_DRIZZLE);
   params->pfx = 1.; /* large pixfrac to not create sudden spatial shifts */
@@ -670,6 +510,7 @@ muse_dar_check(muse_pixtable *aPixtable, double *aMaxShift,
   muse_pixgrid *grid;
   muse_datacube *cube = muse_resampling_cube(aPixtable, params, &grid);
   muse_resampling_params_delete(params);
+  cpl_msg_indent_less();
   if (!cube) {
     muse_pixgrid_delete(grid);
     muse_pixtable_reset_dq(aPixtable, EURO3D_COSMICRAY);
@@ -686,18 +527,19 @@ muse_dar_check(muse_pixtable *aPixtable, double *aMaxShift,
   double lambdaref = cpl_propertylist_get_double(aPixtable->header,
                                                  MUSE_HDR_PT_DAR_NAME),
          lambda1 = (2 - crpix3) * cdelt3 + crval3, /* 2nd plane */
-         lambda3 = (grid->size_z - 1 - crpix3) * cdelt3 + crval3, /* next to last */
+         lambda3 = (grid->nz - 1 - crpix3) * cdelt3 + crval3, /* next to last */
          lambda2 = (lambda1 + lambda3) / 2.; /* central plane */
   if (!cpl_errorstate_is_equal(state) || lambdaref < 0) {
     cpl_errorstate_set(state);
     /* use reference wavelength in the center of the wavelength range */
     lambdaref = lambda2;
-    cpl_msg_warning(__func__, "Data is not DAR corrected, using reference "
-                    "wavelength at %.3f Angstrom!", lambdaref);
     if (!cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_DAR_CORR)) {
       cenhalfsize = 15;
       order = 4;
     }
+    cpl_msg_warning(__func__, "Data is not DAR corrected, using reference "
+                    "wavelength at %.3f Angstrom and polynomial order %d!",
+                    lambdaref, order);
   }
   if (lambdaref > lambda3 || lambdaref < lambda1) { /* ref. outside cube! */
     lambdaref = lambda2;
