@@ -33,11 +33,11 @@
 #include <math.h>
 #include <string.h>
 #include <fenv.h>
-
 #include <cpl.h>
 
-#include "muse_instrument.h"
 #include "muse_lsf.h"
+
+#include "muse_instrument.h"
 #include "muse_optimize.h"
 #include "muse_pfits.h"
 #include "muse_pixtable.h"
@@ -49,7 +49,7 @@
 
 /*----------------------------------------------------------------------------*/
 /**
-   @defgroup muse_lsf       Line Spread Function related functions
+ * @defgroup muse_lsf          Line Spread Function related functions
  */
 /*----------------------------------------------------------------------------*/
 
