@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2015 European Southern Observatory
+ * Copyright (C) 2005-2017 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -736,7 +736,7 @@ muse_pfits_get_out_output_y(const cpl_propertylist *aHeaders,
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the horizontal size of the data region of one quadrant
+  @brief    find out the horizontal size of the data section of one quadrant
   @param    aHeaders       property list/headers to read from
   @param    aQuadrant      quadrant to look for
   @return   the requested value or -1 on error
@@ -757,7 +757,7 @@ muse_pfits_get_out_nx(const cpl_propertylist *aHeaders, unsigned char aQuadrant)
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the vertical size of the data region of one quadrant
+  @brief    find out the vertical size of the data section of one quadrant
   @param    aHeaders       property list/headers to read from
   @param    aQuadrant      quadrant to look for
   @return   the requested value or -1 on error
@@ -1661,6 +1661,24 @@ muse_pfits_get_progid(const cpl_propertylist *aHeaders)
   return value;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the ESO observation target name
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.OBS.TARG.NAME
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_targname(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO OBS TARG NAME");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the value of the flux calibration flag
