@@ -397,7 +397,6 @@ muse_postproc_load_nearest(const cpl_propertylist *aHeader,
 
 /*----------------------------------------------------------------------------*/
 /**
-  @private
   @brief    Revert correction of on-sky data with the flat-field spectrum.
   @param    aPt         pixel table with the on-sky data
   @param    aResponse   the response table with header
@@ -407,13 +406,13 @@ muse_postproc_load_nearest(const cpl_propertylist *aHeader,
   This function is a dirty trick to improve backward compatibility: if an old
   (pre v2.0, without flat-field spectrum correction) STD_RESPONSE was passed to
   the one of the post-processing recipes, then make the (merged) pixel table
-  compatible to that response. by reverting application of the flat-field
+  compatible to that response, by reverting application of the flat-field
   spectrum.
   In effect, this function divides the pixel table again by the averaged
   flat-field spectrum.
  */
 /*----------------------------------------------------------------------------*/
-static cpl_error_code
+cpl_error_code
 muse_postproc_revert_ffspec_maybe(muse_pixtable *aPt,
                                   const muse_table *aResponse)
 {
@@ -432,9 +431,8 @@ muse_postproc_revert_ffspec_maybe(muse_pixtable *aPt,
     return CPL_ERROR_NONE;
   }
 
-  const char *id = "muse_postproc_process_exposure"; /* pretend to be there */
-  cpl_msg_warning(id, "Adapt pixel table to %s for backward compatibility: "
-                  "revert correction by flat-field spectrum!",
+  cpl_msg_warning(__func__, "Adapt pixel table to %s for backward compatibility"
+                  ": revert correction by flat-field spectrum!",
                   MUSE_TAG_STD_RESPONSE);
 
   /* so we need to divide again by the flat-field spectrum */
@@ -449,8 +447,8 @@ muse_postproc_revert_ffspec_maybe(muse_pixtable *aPt,
   cpl_table_delete(aPt->ffspec);
   aPt->ffspec = NULL;
   cpl_propertylist_erase(aPt->header, MUSE_HDR_PT_FFCORR);
-  cpl_msg_info(id, "Pixel table now convolved with flat-field spectrum again, "
-               "removed %s keyword from header.", MUSE_HDR_PT_FFCORR);
+  cpl_msg_info(__func__, "Pixel table now convolved with flat-field spectrum "
+               "again, removed %s keyword from header.", MUSE_HDR_PT_FFCORR);
   return CPL_ERROR_NONE;
 } /* muse_postproc_revert_ffspec_maybe() */
 
