@@ -254,7 +254,10 @@ muse_processing_prepare_property(cpl_propertylist *aHeader, const char *aName,
     // here:
     // If it is not set (the normal case in a recipe), just an error
     // message is generated in case of a missing header.
-    if (!cpl_propertylist_has(aHeader, "MUSE PRIVATE DOCUMENTATION")) {
+    // If the description contains the string "(optional)", the property may not
+    // be present, but this failure should be ignored.
+    if (!cpl_propertylist_has(aHeader, "MUSE PRIVATE DOCUMENTATION") &&
+        (aDescription && !strstr(aDescription, "(optional)"))) {
       cpl_msg_warning(__func__, "Property %s (%s) not used", aName,
                       aDescription);
       return CPL_ERROR_DATA_NOT_FOUND;
@@ -262,12 +265,14 @@ muse_processing_prepare_property(cpl_propertylist *aHeader, const char *aName,
     // If this property is set (if the function is used from
     // muse_xmldoc), then we add missing headers with some default
     // values.
+    // (For floating-point keywords, use "curious" values, since the better
+    // choice of NAN cannot be written to FITS headers and causes an error.)
     switch(aType) {
     case CPL_TYPE_FLOAT:
-        cpl_propertylist_append_float(aHeader, aName, NAN);
+        cpl_propertylist_append_float(aHeader, aName, -99.);
         break;
       case CPL_TYPE_DOUBLE:
-        cpl_propertylist_append_double(aHeader, aName, NAN);
+        cpl_propertylist_append_double(aHeader, aName, -999.);
         break;
       case CPL_TYPE_STRING:
         cpl_propertylist_append_string(aHeader, aName, "");
@@ -327,9 +332,9 @@ muse_processing_prepare_property(cpl_propertylist *aHeader, const char *aName,
   @param  aPlugin   the CPL plugin of the recipe in question
 
   Output pipeline name (PACKAGE_NAME) and version (PACKAGE_VERSION) and if
-  aPlugin is non-NULL and the message or log levels are set to debug, give all
-  input frames (filenames and tags), non-default parameters, and relevant
-  environment variables.
+  aPlugin is non-NULL, give all input frames (filenames and tags), non-default
+  parameters, and relevant (MUSE and OpenMP) environment variables.
+  @note Only known environment variables are tested.
 
   This function is meant to be called at the beginning of a MUSE recipe.
  */
@@ -342,12 +347,12 @@ muse_processing_recipeinfo(cpl_plugin *aPlugin)
   if (!aPlugin) {
     return;
   }
-  if (cpl_msg_get_level() != CPL_MSG_DEBUG &&
-      cpl_msg_get_log_level() != CPL_MSG_DEBUG) {
-    return;
-  }
   cpl_recipe *recipe = (cpl_recipe *)aPlugin;
+  cpl_errorstate state = cpl_errorstate_get();
   cpl_size nframes = cpl_frameset_get_size(recipe->frames);
+  if (!cpl_errorstate_is_equal(state)) {
+    cpl_errorstate_set(state); /* swallow error about missing frames */
+  }
   cpl_msg_debug(__func__, "%"CPL_SIZE_FORMAT" input frames:", nframes);
 
   cpl_msg_indent_more();
@@ -418,19 +423,64 @@ muse_processing_recipeinfo(cpl_plugin *aPlugin)
   }
   cpl_msg_indent_less();
 
-  cpl_msg_debug(__func__, "relevant environment variables:");
+  /* print known MUSE* environment variables, keep this in sync with README */
+  cpl_msg_debug(__func__, "relevant MUSE environment variables:");
   cpl_msg_indent_more();
-  /* print all MUSE* environment variables */
-  /* XXX not sure how to do that in a portable way, so take a few important ones */
-  char *expuser = getenv("MUSE_EXPERT_USER");
-  if (expuser) {
-    cpl_msg_debug(__func__, "MUSE_EXPERT_USER=%s", expuser);
-  }
-  /* print OMP_NUM_THREADS variable */
-  char *omp_num_threads = getenv("OMP_NUM_THREADS");
-  if (omp_num_threads) {
-    cpl_msg_debug(__func__, "OMP_NUM_THREADS=%s", omp_num_threads);
-  }
+  const char *env[] = { "MUSE_AIT_HACK_SLICE_NUMBER",
+                        "MUSE_BASICPROC_SKIP_GAIN_OVERRIDE",
+                        "MUSE_BASICPROC_SKIP_NONLIN_CORR",
+                        "MUSE_TWILIGHT_SCALES", "MUSE_TWILIGHT_SKIP",
+                        "MUSE_GEOMETRY_SKIP", "MUSE_GEOMETRY_NO_INVERSION",
+                        "MUSE_GEOMETRY_MASK_ROTATION",
+                        "MUSE_GEOMETRY_PINHOLE_DY", "MUSE_GEOMETRY_STD_GAP",
+                        "MUSE_GEOMETRY_HORI_OFFSETS", "MUSE_DAR_CORRECT_METHOD",
+                        "MUSE_PIXTABLE_SAVE_AS_TABLE", "MUSE_COLLAPSE_PIXTABLE",
+                        "MUSE_COLLAPSE_USE_VARIANCE",
+                        "MUSE_CPL_ERRORSTATE_NDUMP", "MUSE_EXPERT_USER",
+                        "MUSE_BADPIX_FROM_MASKCUBE", "MUSE_DEBUG_QUADRANTS",
+                        "MUSE_DEBUG_TRACE", "MUSE_DEBUG_WAVECAL",
+                        "MUSE_DEBUG_LSF_FIT", "MUSE_DEBUG_SKY",
+                        "MUSE_PLOT_TRACE", "MUSE_DEBUG_DCR", "MUSE_DEBUG_FLUX",
+                        "MUSE_DEBUG_PIXTABLE_LIMITS",
+                        "MUSE_DEBUG_GEO_VERIFY_DY", "MUSE_DEBUG_GEO_VERTICAL",
+                        "MUSE_DEBUG_MEMORY_PROGRAM", "MUSE_DEBUG_WCS",
+                        "MUSE_DEBUG_GRID_CONVERSION", "MUSE_DEBUG_NEAREST",
+                        "MUSE_DEBUG_WEIGHTED", "MUSE_DEBUG_WEIGHTED_X",
+                        "MUSE_DEBUG_WEIGHTED_Y", "MUSE_DEBUG_WEIGHTED_Z",
+                        "MUSE_DEBUG_WEIGHTED_GRID", "MUSE_DEBUG_WEIGHT_CUBE",
+                        "MUSE_DEBUG_CRREJECT", "MUSE_DEBUG_CRREJECT_X",
+                        "MUSE_DEBUG_CRREJECT_Y", "MUSE_DEBUG_CRREJECT_Z",
+                        NULL };
+  int ienv = 0;
+  while (env[ienv]) {
+    char *value = getenv(env[ienv]);
+    if (value) {
+      cpl_msg_debug(__func__, "%s=%s", env[ienv], value);
+    } /* if */
+    ienv++;
+  } /* while */
+  cpl_msg_indent_less();
+
+  /* print OpenMP-related variables */
+  cpl_msg_debug(__func__, "relevant OpenMP environment variables:");
+  cpl_msg_indent_more();
+  /* list all variables from OpenMP spec v3.1, section 4, add    *
+   * those that are also part of OpenMP v4.0 and v4.5 at the end */
+  const char *envomp[] = { "OMP_SCHEDULE", "OMP_NUM_THREADS", "OMP_DYNAMIC",
+                           "OMP_PROC_BIND", "OMP_NESTED", "OMP_STACKSIZE",
+                           "OMP_WAIT_POLICY", "OMP_MAX_ACTIVE_LEVELS",
+                           "OMP_THREAD_LIMIT", /* v3.1 */
+                           "OMP_PLACES", "OMP_CANCELLATION", "OMP_DISPLAY_ENV",
+                           "OMP_DEFAULT_DEVICE", /* v4.0 */
+                           "OMP_MAX_TASK_PRIORITY", /* v4.5 */ NULL };
+  ienv = 0;
+  while (envomp[ienv]) {
+    char *value = getenv(envomp[ienv]);
+    if (value) {
+      cpl_msg_debug(__func__, "%s=%s", envomp[ienv], value);
+    } /* if */
+    ienv++;
+  } /* while */
   cpl_msg_indent_less();
 } /* muse_processing_recipeinfo() */
 
