@@ -29,6 +29,7 @@
 
 #include "muse_image.h"
 #include "muse_imagelist.h"
+#include "muse_table.h"
 
 /*----------------------------------------------------------------------------*
  *                          Special variable types                            *
@@ -99,7 +100,7 @@ void muse_wave_params_delete(muse_wave_params *);
 cpl_table *muse_wave_calib(muse_image *, cpl_table *, cpl_table *, muse_wave_params *);
 cpl_table *muse_wave_calib_lampwise(muse_imagelist *, cpl_table *, cpl_table *, muse_wave_params *);
 
-cpl_boolean muse_wave_lines_check(cpl_table *, cpl_propertylist *);
+cpl_boolean muse_wave_lines_check(muse_table *);
 cpl_vector *muse_wave_lines_get(cpl_table *, int, double);
 cpl_vector *muse_wave_lines_get_for_lamp(cpl_table *, const char *, int, double);
 const char *muse_wave_lines_get_lampname(cpl_table *, const int);
