@@ -100,6 +100,9 @@ muse_create_sky_load_pixtable(muse_processing *aProcessing,
       muse_pixtable_delete(pixtable);
       pixtable = NULL;
     } else {
+      /* See if we need to revert the flat-field spectrum correction, *
+       * to match pixel table and response curve, before applying it. */
+      muse_postproc_revert_ffspec_maybe(pixtable, response);
       cpl_error_code rc = muse_flux_calibrate(pixtable, response, extinction,
                                               telluric);
       if (rc != CPL_ERROR_NONE) {
