@@ -469,6 +469,279 @@ muse_align_celestial_from_pixel(cpl_table *aTable,
   return 0;
 }
 
+/* XXX: Keep the conditional until the SVD code has been fully verified. *
+ *      Once this is done, completely replace the old code with the SVD  *
+ *      version.                                                         */
+#define USE_SVD_SOLVER
+#ifdef USE_SVD_SOLVER
+
+#if CPL_VERSION_CODE < 459008
+#define cpl_matrix_solve_svd muse_cplmatrix_solve_svd
+
+/*
+ * The following implementation of one-sided Jacobi orthogonalization
+ * to compute the SVD of a MxN matrix, with M >= N is based on the
+ * implementation found in the GNU Scientific Library (GSL).
+ */
+
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief    Compute the singular value decomposition of a real matrix using
+            Jacobi plane rotations.
+  @param U  The M x N matrix to be factorized.
+  @param S  A vector of N elements for storing the singular values.
+  @param V  An N x N square matrix for storing the right singular vectors.
+  @return   @c CPL_ERROR_NONE if the factorization was successful and an error
+            code otherwise.
+ 
+  @error{set CPL_ERROR_NULL_INPUT\, return CPL_ERROR_NULL_INPUT,
+         One or more arguments are NULL}
+  @error{set CPL_ERROR_ILLEGAL_INPUT\, return CPL_ERROR_ILLEGAL_INPUT,
+         the matrix U has less rows than columns (M < N), or the matrix V
+         is not a square matrix}
+  @error{set CPL_ERROR_INCOMPATIBLE_INPUT\, return CPL_ERROR_INCOMPATIBLE_INPUT,
+         the number of rows of the matrix V does not match the number
+         of columns of U (N), or the size of the vector S does not match
+         the number of rows of U (M)}
+ 
+  The function computes singular value decomposition (SVD) of a real M x N
+  matrix @em U, where M >= N. It uses a one-sided Jacobi orthogonalization
+  as described in James Demmel, Kresimir Veselic, "Jacobi's Method is more
+  accurate than QR", Lapack Working Note 15 (LAWN15), October 1989,
+  Algorithm 4.1 which is available at @em netlib.org.
+
+  The input matrix @em U is overwritten during the process and contains
+  in the end the left singular vectors of @em U. The matrix @em V will be
+  updated with the right singular vectors of @em U, and the vector @em S
+  will contain the singular values. For efficiency reasons the singular
+  values are not written to the diagonal elements of a N x N diagonal matrix,
+  but an N-elements vector.
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_error_code
+muse_cplmatrix_decomp_sv_jacobi(cpl_matrix *U, cpl_vector *S, cpl_matrix *V)
+{
+  cpl_ensure_code((U != NULL) && (V != NULL) && (S != NULL),
+                  CPL_ERROR_NULL_INPUT);
+  cpl_ensure_code(cpl_matrix_get_nrow(U) >= cpl_matrix_get_ncol(U),
+                  CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(cpl_matrix_get_nrow(V) == cpl_matrix_get_ncol(V),
+                  CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(cpl_matrix_get_nrow(V) == cpl_matrix_get_ncol(U),
+                  CPL_ERROR_INCOMPATIBLE_INPUT);
+  cpl_ensure_code(cpl_vector_get_size(S) == cpl_matrix_get_ncol(U),
+                  CPL_ERROR_INCOMPATIBLE_INPUT);
+
+  const cpl_size m = cpl_matrix_get_nrow(U);
+  const cpl_size n = cpl_matrix_get_ncol(U);
+
+  /* Initialize the rotation counter and the sweep counter. *
+   * Use a minimum number of 12 sweeps.                     */
+  cpl_size count = 1;
+  cpl_size sweep = 0;
+  cpl_size sweepmax = CPL_MAX(5 * n, 12);
+
+  double tolerance = 10. * m * DBL_EPSILON;
+
+  /* Fill V with the identity matrix. */
+  cpl_matrix_fill(V, 0.);
+  cpl_matrix_fill_diagonal(V, 1., 0);
+
+  /* Store the column error estimates in S, for use during the *
+   * orthogonalization.                                        */
+
+  cpl_size i;
+  cpl_size j;
+  cpl_vector *cv1 = cpl_vector_new(m);
+  double *const _cv1 = cpl_vector_get_data(cv1);
+
+  for (j = 0; j < n; ++j) {
+    for (i = 0; i < m; ++i) {
+      _cv1[i] = cpl_matrix_get(U, i, j);
+    }
+
+    double s = sqrt(cpl_vector_product(cv1, cv1));
+    cpl_vector_set(S, j, DBL_EPSILON * s);
+  }
+
+  /* Orthogonalization of U by plane rotations. */
+  cpl_vector *cv2 = cpl_vector_new(m);
+  double *const _cv2 = cpl_vector_get_data(cv2);
+
+  while ((count > 0) && (sweep <= sweepmax)) {
+
+    /* Initialize the rotation counter. */
+    count = n * (n - 1) / 2;
+
+    for (j = 0; j < n - 1; ++j) {
+      cpl_size k;
+      for (k = j + 1; k < n; ++k) {
+        for (i = 0; i < m; ++i) {
+          _cv1[i] = cpl_matrix_get(U, i, j);
+        }
+        for (i = 0; i < m; ++i) {
+          _cv2[i] = cpl_matrix_get(U, i, k);
+        }
+
+        /* Compute the (j, k) submatrix elements of Ut*U. The *
+         * non-diagonal element c is scaled by 2.             */
+        double a = sqrt(cpl_vector_product(cv1, cv1));
+        double b = sqrt(cpl_vector_product(cv2, cv2));
+        double c = 2. * cpl_vector_product(cv1, cv2);
+
+        /* Test whether columns j, k are orthogonal, or dominant errors. */
+        double abserr_a = cpl_vector_get(S, j);
+        double abserr_b = cpl_vector_get(S, k);
+        cpl_boolean orthogonal = (fabs(c) <= tolerance * a * b);
+        cpl_boolean sorted     = (a >= b);
+        cpl_boolean noisy_a    = (a < abserr_a);
+        cpl_boolean noisy_b    = (b < abserr_b);
+
+        if (sorted && (orthogonal || noisy_a || noisy_b)) {
+          --count;
+          continue;
+        }
+
+        /* Compute the Jacobi rotation which diagonalizes the (j, k) *
+         * submatrix.                                                */
+        double q = a * a - b * b;
+        double v = sqrt(c * c + q * q);
+        double cs;  /* cos(theta) */
+        double sn;  /* sin(theta) */
+
+        if (v == 0. || !sorted) {
+          cs = 0.;
+          sn = 1.;
+        } else {
+          cs = sqrt((v + q) / (2. * v));
+          sn = c / (2. * v * cs);
+        }
+
+        /* Update the columns j and k of U and V */
+        for (i = 0; i < m; ++i) {
+          const double u_ij = cpl_matrix_get(U, i, j);
+          const double u_ik = cpl_matrix_get(U, i, k);
+          cpl_matrix_set(U, i, j,  u_ij * cs + u_ik * sn);
+          cpl_matrix_set(U, i, k, -u_ij * sn + u_ik * cs);
+        }
+
+        for (i = 0; i < n; ++i) {
+          const double v_ij = cpl_matrix_get(V, i, j);
+          const double v_ik = cpl_matrix_get(V, i, k);
+          cpl_matrix_set(V, i, j,  v_ij * cs + v_ik * sn);
+          cpl_matrix_set(V, i, k, -v_ij * sn + v_ik * cs);
+        }
+
+        cpl_vector_set(S, j, fabs(cs) * abserr_a +
+            fabs(sn) * abserr_b);
+        cpl_vector_set(S, k, fabs(sn) * abserr_a +
+            fabs(cs) * abserr_b);
+      }
+    }
+    ++sweep;
+
+  }
+  cpl_vector_delete(cv2);
+
+  /* Compute singular values. */
+  double prev_norm = -1.;
+  for (j = 0; j < n; ++j) {
+    for (i = 0; i < m; ++i) {
+      _cv1[i] = cpl_matrix_get(U, i, j);
+    }
+
+    double norm = sqrt(cpl_vector_product(cv1, cv1));
+
+    /* Determine if the singular value is zero, according to the  *
+     * criteria used in the main loop above (i.e. comparison with *
+     * norm of previous column).                                  */
+    if ((norm == 0.) || (prev_norm == 0.) ||
+        ((j > 0) && (norm <= tolerance * prev_norm))) {
+      cpl_vector_set(S, j, 0.);          /* singular */
+      cpl_matrix_fill_column(U, 0., j);  /* annihilate column */
+      prev_norm = 0.;
+    } else {
+      cpl_vector_set(S, j, norm);     /* non-singular */
+
+      /* Normalize the column vector of U */
+      for (i = 0; i < m; ++i) {
+        cpl_matrix_set(U, i, j, _cv1[i] / norm);
+      }
+      prev_norm = norm;
+    }
+  }
+  cpl_vector_delete(cv1);
+
+  if (count > 0) {
+    return CPL_ERROR_CONTINUE;
+  }
+  return CPL_ERROR_NONE;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief  Solve a linear system in a least square sense using an
+          SVD factorization.
+  @param  coeff Linear system coefficients
+  @param  rhs   Right hand side of the linear system
+  @return the solution vector or NULL on error
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_matrix *
+muse_cplmatrix_solve_svd(const cpl_matrix *coeff, const cpl_matrix *rhs)
+{
+  cpl_ensure((coeff != NULL) && (rhs != NULL), CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(cpl_matrix_get_nrow(coeff) == cpl_matrix_get_nrow(rhs),
+             CPL_ERROR_INCOMPATIBLE_INPUT, NULL);
+  cpl_ensure(cpl_matrix_get_ncol(rhs) == 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
+
+  /* Compute the SVD factorization of the coefficient matrix */
+  cpl_matrix *U = cpl_matrix_duplicate(coeff);
+  cpl_size nc = cpl_matrix_get_ncol(U);
+
+  cpl_matrix *V = cpl_matrix_new(nc, nc);
+  cpl_vector *S = cpl_vector_new(nc);
+
+  cpl_error_code status = muse_cplmatrix_decomp_sv_jacobi(U, S, V);
+  if (status != CPL_ERROR_NONE) {
+    cpl_vector_delete(S);
+    cpl_matrix_delete(V);
+    cpl_matrix_delete(U);
+    cpl_error_set_where(cpl_func);
+    return NULL;
+  }
+
+  /* Solve the linear system */
+  cpl_matrix *Ut = cpl_matrix_transpose_create(U);
+  cpl_matrix *w  = cpl_matrix_product_create(Ut, rhs);
+  cpl_matrix_delete(Ut);
+
+  const double *_s = cpl_vector_get_data_const(S);
+  double *_w = cpl_matrix_get_data(w);
+  cpl_size ic;
+
+  for (ic = 0; ic < nc; ++ic) {
+    double s = _s[ic];
+    if (s != 0.) {
+      s = 1. / s;
+    }
+    _w[ic] *= s;
+  }
+
+  cpl_matrix *solution = cpl_matrix_product_create(V, w);
+  cpl_matrix_delete(w);
+  cpl_vector_delete(S);
+  cpl_matrix_delete(V);
+  cpl_matrix_delete(U);
+  return solution;
+}
+
+#endif
+
+#endif
+
 /*----------------------------------------------------------------------------*/
 /**
   @private
@@ -493,6 +766,7 @@ muse_cplmatrix_solve_least_square(const cpl_matrix *aMatrix1,
   cpl_matrix *result = NULL;
   cpl_errorstate status = cpl_errorstate_get();
 
+#ifndef USE_SVD_SOLVER
   if (nr == nc) {
     result = cpl_matrix_solve(aMatrix1, aMatrix2);
   } else {
@@ -510,6 +784,20 @@ muse_cplmatrix_solve_least_square(const cpl_matrix *aMatrix1,
       cpl_matrix_delete(mt);
     }
   }
+#else
+  if (nr >= nc) {
+    result = cpl_matrix_solve_svd(aMatrix1, aMatrix2);
+  } else {
+    cpl_matrix *mt = cpl_matrix_transpose_create(aMatrix1);
+    cpl_matrix *mgram = cpl_matrix_product_create(aMatrix1, mt);
+    cpl_matrix *mw = cpl_matrix_solve(mgram, aMatrix2);
+
+    result = cpl_matrix_product_create(mt, mw);
+    cpl_matrix_delete(mw);
+    cpl_matrix_delete(mgram);
+    cpl_matrix_delete(mt);
+  }
+#endif
 
   if (status != cpl_errorstate_get()) {
     cpl_matrix_delete(result);
