@@ -49,6 +49,7 @@
   @param    aGeo          geometry table
   @param    aImage        actual pre-reduced CCD image of the science data
   @param    aSkyLines     array of sky line wavelengths
+  @param    aFlat         image of the relevant master-flat field
   @param    aAttached     table of per-slice corrections from an attached flat
   @param    aTwilights    NULL-terminated list of twilight cubes
   @return   0 if everything is ok, -1 something went wrong
@@ -62,7 +63,8 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
                            muse_scibasic_params_t *aParams,
                            cpl_table *aTrace, cpl_table *aWave, cpl_table *aGeo,
                            muse_image *aImage, cpl_array *aSkyLines,
-                           cpl_table *aAttached, muse_datacube **aTwilights)
+                           muse_image *aFlat, cpl_table *aAttached,
+                           muse_datacube **aTwilights)
 {
   cpl_ensure(aImage && aTrace && aWave, CPL_ERROR_NULL_INPUT, -1);
 
@@ -87,9 +89,28 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
     cpl_free(intag);
     return -1;
   }
+  muse_pixtable_append_ff(pixtable, aFlat, aTrace, aWave, kMuseSpectralSamplingA);
   if (aSkyLines) {
+    cpl_array *rejpars = muse_cplarray_new_from_delimited_string(aParams->skyreject, ",");
+    int nrej = cpl_array_get_size(rejpars);
+    float lsigma = 15.,
+          hsigma = 15.;
+    unsigned char niter = 1;
+    if (nrej > 0 && cpl_array_get_string(rejpars, 0)) {
+      /* first number: high sigma limit */
+      hsigma = atof(cpl_array_get_string(rejpars, 0));
+    }
+    if (nrej > 1 && cpl_array_get_string(rejpars, 1)) {
+      /* second number: low sigma limit */
+      lsigma = atof(cpl_array_get_string(rejpars, 1));
+    }
+    if (nrej > 2 && cpl_array_get_string(rejpars, 2)) {
+      /* second number: low sigma limit */
+      niter = atoi(cpl_array_get_string(rejpars, 2));
+    }
+    cpl_array_delete(rejpars);
     muse_basicproc_shift_pixtable(pixtable, aSkyLines, aParams->skyhalfwidth,
-                                  aParams->skybinsize);
+                                  aParams->skybinsize, lsigma, hsigma, niter);
   }
 
   if (aParams->resample) {
@@ -159,6 +180,7 @@ muse_scibasic_compute(muse_processing *aProcessing,
                       muse_scibasic_params_t *aParams)
 {
   muse_imagelist *images = NULL;
+  muse_image *masterflat = NULL;
   if (muse_processing_check_intags(aProcessing, "REDUCED", 8)) {
     cpl_msg_warning(__func__, "Found REDUCED files on input, ignoring all "
                     "others inputs!");
@@ -173,16 +195,19 @@ muse_scibasic_compute(muse_processing *aProcessing,
   } else {
     muse_basicproc_params *bpars = muse_basicproc_params_new(aProcessing->parameters,
                                                              "muse.muse_scibasic");
+    bpars->keepflat = CPL_TRUE;
     images = muse_basicproc_load(aProcessing, aParams->nifu, bpars);
+    masterflat = bpars->flatimage;
+    bpars->flatimage = NULL; /* so that it doesn't get deallocated */
     muse_basicproc_params_delete(bpars);
   }
   cpl_ensure(images, cpl_error_get_code(), -1);
 
-  cpl_table *tracetable = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE,
-                                          aParams->nifu);
-  cpl_table *wavecaltable = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE,
-                                            aParams->nifu);
-  cpl_table *geotable = muse_table_load(aProcessing, MUSE_TAG_GEOMETRY_TABLE, 0);
+  cpl_table *tracetable = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE,
+                                                      aParams->nifu);
+  cpl_table *wavecaltable = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE,
+                                                        aParams->nifu);
+  cpl_table *geotable = muse_processing_load_ctable(aProcessing, MUSE_TAG_GEOMETRY_TABLE, 0);
   if (!tracetable || !wavecaltable || !geotable) {
     cpl_msg_error(__func__, "Calibration could not be loaded for IFU %d:%s%s%s",
                   aParams->nifu, !tracetable ? " "MUSE_TAG_TRACE_TABLE : "",
@@ -236,7 +261,7 @@ muse_scibasic_compute(muse_processing *aProcessing,
       muse_image *image = muse_imagelist_get(images, k);
       rc = muse_scibasic_per_exposure(aProcessing, aParams, tracetable,
                                       wavecaltable, geotable, image, skylines,
-                                      tattached, twilights);
+                                      masterflat, tattached, twilights);
     } /* for k (all science images) */
     cpl_table_delete(tattached);
   } else { /* some combination is supposed to happen */
@@ -254,9 +279,10 @@ muse_scibasic_compute(muse_processing *aProcessing,
                                    cpl_array_get_string(aProcessing->intags, 0));
     rc = muse_scibasic_per_exposure(aProcessing, aParams, tracetable,
                                     wavecaltable, geotable, image, skylines,
-                                    NULL, twilights);
+                                    masterflat, NULL, twilights);
     muse_image_delete(image);
   } /* else: combined processing */
+  muse_image_delete(masterflat);
   cpl_array_delete(skylines);
   muse_combinepar_delete(cpars);
 
