@@ -154,12 +154,15 @@ muse_scipost_compute(muse_processing *aProcessing,
   }
   prop->rvtype = muse_rvcorrect_select_type(aParams->rvcorr_s);
   /* flux calibration */
-  prop->response = muse_table_load(aProcessing, MUSE_TAG_STD_RESPONSE, 0);
-  prop->extinction = muse_table_load(aProcessing, MUSE_TAG_EXTINCT_TABLE, 0);
-  prop->telluric = muse_table_load(aProcessing, MUSE_TAG_STD_TELLURIC, 0);
+  prop->response = muse_processing_load_table(aProcessing,
+                                              MUSE_TAG_STD_RESPONSE, 0);
+  prop->telluric = muse_processing_load_table(aProcessing,
+                                              MUSE_TAG_STD_TELLURIC, 0);
+  prop->extinction = muse_processing_load_ctable(aProcessing,
+                                                 MUSE_TAG_EXTINCT_TABLE, 0);
   /* astrometric correction */
   if (aParams->astrometry) {
-    prop->wcs = muse_propertylist_load(aProcessing, MUSE_TAG_ASTROMETRY_WCS);
+    prop->wcs = muse_processing_load_header(aProcessing, MUSE_TAG_ASTROMETRY_WCS);
     if (!prop->wcs) {
       /* if it was not in the input, create a default WFM astrometry */
       cpl_msg_warning(__func__, "Using default MUSE WFM astrometry, output "
@@ -253,7 +256,10 @@ muse_scipost_compute(muse_processing *aProcessing,
       return -1;
     }
     prop->sky_continuum = muse_sky_continuum_load(aProcessing);
-    prop->sky_mask = muse_processing_mask_load(aProcessing, MUSE_TAG_SKY_MASK);
+    prop->sky_mask = muse_processing_load_mask(aProcessing, MUSE_TAG_SKY_MASK);
+  } else if (aParams->skymethod == MUSE_SCIPOST_PARAM_SKYMETHOD_SIMPLE) {
+    /* a mask could help for the simple sky subtraction */
+    prop->sky_mask = muse_processing_load_mask(aProcessing, MUSE_TAG_SKY_MASK);
   }
 
   /* sort input pixel tables into different exposures */
@@ -266,7 +272,7 @@ muse_scipost_compute(muse_processing *aProcessing,
   int nexposures = cpl_table_get_nrow(prop->exposures);
 
   /* load optional offsets */
-  cpl_table *offsets = muse_table_load(aProcessing, MUSE_TAG_OFFSET_LIST, 0);
+  cpl_table *offsets = muse_processing_load_ctable(aProcessing, MUSE_TAG_OFFSET_LIST, 0);
   if (offsets && muse_cpltable_check(offsets, muse_offset_list_def) != CPL_ERROR_NONE) {
     cpl_msg_warning(__func__, "Input %s has unexpected format, proceeding "
                     "without offset and flux scales!", MUSE_TAG_OFFSET_LIST);
