@@ -94,6 +94,11 @@ muse_twilight_qc_header(muse_image *aImage, muse_imagelist *aList)
   interpolated, using @cpl_detector_interpolate_rejected(), before a pixel
   table is created for the IFU using @ref muse_pixtable_create().
 
+  @note This function ignores the flat-field spectrum approach of the on-sky
+        data (see muse_scibasic, muse_scipost etc.) since the twilight cube
+        correction gets applied to the pixel tables before the flat-field
+        spectrum is taken out.
+
   All pixel tables are then restricted to the wavelength range given in aParams,
   and their flux is summed up and recorded in the MUSE_HDR_FLAT_FLUX_SKY"%hhu"
   keyword. This relative flux is then used to scale all pixel tables to a common
@@ -130,7 +135,7 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
          lmax = aParams->lambdamax;
   cpl_ensure(lmax > lmin, CPL_ERROR_ILLEGAL_INPUT, NULL);
   /* the geometry table is common for all IFUs */
-  cpl_table *geo = muse_table_load(aProcessing, MUSE_TAG_GEOMETRY_TABLE, 0);
+  cpl_table *geo = muse_processing_load_ctable(aProcessing, MUSE_TAG_GEOMETRY_TABLE, 0);
   cpl_ensure(geo, CPL_ERROR_FILE_NOT_FOUND, NULL);
 
   muse_pixtable **pts = cpl_calloc(kMuseNumIFUs, sizeof(muse_pixtable));
@@ -138,8 +143,8 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
   #pragma omp parallel for default(none)                 /* as req. by Ralf */ \
           shared(aBPars, aCPars, aProcessing, geo, pts)
   for (nifu = 1; nifu <= kMuseNumIFUs; nifu++) {
-    cpl_table *trace = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
-              *wavecal = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
+    cpl_table *trace = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
+              *wavecal = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
     if (!trace || !wavecal) {
       cpl_msg_warning(__func__, "Calibrations could not be loaded for IFU "
                       "%2hhu:%s%s", nifu, !trace ? " "MUSE_TAG_TRACE_TABLE : "",
@@ -522,7 +527,7 @@ muse_twilight_compute(muse_processing *aProcessing,
   bpm = bpm2; /* easier handle to remember */
 
   /* apply a mask, if one was given, on top of the one that the image already has */
-  muse_mask *maskimage = muse_processing_mask_load(aProcessing,
+  muse_mask *maskimage = muse_processing_load_mask(aProcessing,
                                                    MUSE_TAG_VIGN_MASK);
   /* adapt the mask to the size of the actual cube */
   if (maskimage) {
