@@ -103,7 +103,7 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
 
   /* create built-in white-light filter, luckily it doesn't *
    * need the processing argument to "load" this one        */
-  cpl_table *fwhite = muse_table_load_filter(NULL, "white");
+  muse_table *fwhite = muse_table_load_filter(NULL, "white");
 
   /* duplicate the pixel table, project to sky, to also get resampling with WCS */
   muse_pixtable *ptdeg = muse_pixtable_duplicate(aPtPix);
@@ -155,12 +155,101 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
           cpl_msg_debug(__func__, "weight image mean = %f", mean);
           cpl_test(mean > 0.3 && mean < 1.0); /* some rough guess */
           cpl_test(min >= 0.); /* zero or positive weights */
+          int xsize = cpl_image_get_size_x(plane),
+              ysize = cpl_image_get_size_y(plane);
           cpl_image_delete(plane);
           cpl_test(unsetenv("MUSE_DEBUG_WEIGHT_CUBE") == 0);
 #if MUSE_TEST_RESAMPLING_DELETE_FILES
           cpl_test(remove(fnw) == 0); /* can delete the file again */
 #endif
-        }
+
+          /* test muse_pixgrid_create() for a few cases of unusual sizes */
+          cpl_errorstate state;
+          int ig;
+          for (ig = 0; ig < 20; ig++) {
+            cpl_msg_debug(__func__, "unusual z-size %d", ig);
+            state = cpl_errorstate_get();
+            muse_pixgrid *gridtest = muse_pixgrid_create(pt, cube->header,
+                                                         xsize, ysize, ig);
+            if (ig == 0) {
+              cpl_test(!cpl_errorstate_is_equal(state) &&
+                       cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+              cpl_errorstate_set(state);
+              cpl_test_null(gridtest);
+              continue;
+            }
+            cpl_test(cpl_errorstate_is_equal(state));
+            cpl_test_nonnull(gridtest);
+            muse_pixgrid_delete(gridtest);
+          } /* for ig (some unsual z-axis sizes) */
+          /* while we are here, also check other failure cases of muse_pixgrid_create() */
+          if (tl == MUSE_RESAMPLING_DISP_AWAV) {
+            state = cpl_errorstate_get();
+            muse_pixgrid *gridtest = muse_pixgrid_create(NULL, cube->header,
+                                                         xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            gridtest = muse_pixgrid_create(pt, cube->header, 0, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            gridtest = muse_pixgrid_create(pt, cube->header, xsize, 0, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            muse_pixtable *pt33 = muse_pixtable_duplicate(pt);
+            cpl_table_set_column_unit(pt33->table, MUSE_PIXTABLE_XPOS, "rad");
+            cpl_table_set_column_unit(pt33->table, MUSE_PIXTABLE_YPOS, "rad");
+            gridtest = muse_pixgrid_create(pt33, cube->header, xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_UNSUPPORTED_MODE);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            cpl_table_set_column_unit(pt33->table, MUSE_PIXTABLE_XPOS, "deg");
+            cpl_table_set_column_unit(pt33->table, MUSE_PIXTABLE_YPOS, "deg");
+            cpl_table_set_size(pt33->table, 0);
+            gridtest = muse_pixgrid_create(pt33, cube->header, xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            muse_pixtable_delete(pt33);
+            /* missing x/y columns cause the above "unsupported mode" error */
+            pt33 = muse_pixtable_duplicate(pt);
+            cpl_table_erase_column(pt33->table, MUSE_PIXTABLE_LAMBDA);
+            gridtest = muse_pixgrid_create(pt33, cube->header, xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND);
+            cpl_errorstate_set(state);
+            cpl_test_null(gridtest);
+            muse_pixtable_delete(pt33);
+            /* finally, create pixel tables that each have one extreme *
+             * pixel that is missed in the grid construction; this is  *
+             * constructed to fall beyond the -INT_MAX/2 limit...      */
+            pt33 = muse_pixtable_duplicate(pt);
+            cpl_table_set_float(pt33->table, MUSE_PIXTABLE_LAMBDA, 0, -1342171050.);
+            gridtest = muse_pixgrid_create(pt33, cube->header, xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_ILLEGAL_OUTPUT);
+            cpl_errorstate_set(state);
+            cpl_test_nonnull(gridtest);
+            muse_pixgrid_delete(gridtest);
+            muse_pixtable_delete(pt33);
+            pt33 = muse_pixtable_duplicate(pt);
+            cpl_table_set_float(pt33->table, MUSE_PIXTABLE_LAMBDA, 0, 1342184800.);
+            gridtest = muse_pixgrid_create(pt33, cube->header, xsize, ysize, 5);
+            cpl_test(!cpl_errorstate_is_equal(state) &&
+                     cpl_error_get_code() == CPL_ERROR_ILLEGAL_OUTPUT);
+            cpl_errorstate_set(state);
+            cpl_test_nonnull(gridtest);
+            muse_pixgrid_delete(gridtest);
+            muse_pixtable_delete(pt33);
+          } /* if tl = awav */
+        } /* for t = drizzle */
         cpl_free(fnw);
         /* primitive tests */
         cpl_test_nonnull(cube);
@@ -593,6 +682,53 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
           cpl_errorstate_set(state);
         } /* if drizzle */
 
+        /* for one of the tests, also create a mask cube and test the bad pixel table creation this way */
+        if (t == MUSE_RESAMPLE_WEIGHTED_DRIZZLE &&
+            tl == MUSE_RESAMPLING_DISP_AWAV && ipt == 0) {
+          setenv("MUSE_BADPIX_FROM_MASKCUBE",
+                 "maskcube_test.fits,badpix_test.fits", 1);
+          /* create the mask cube */
+          int nx = cpl_image_get_size_x(cpl_imagelist_get(cube->data, 0)),
+              ny = cpl_image_get_size_y(cpl_imagelist_get(cube->data, 0));
+          cpl_image *mask = cpl_image_new(nx, ny, CPL_TYPE_INT);
+          cpl_image_fill_window(mask, 8, 3, 12, 5, 1);
+          cpl_imagelist *maskcube = cpl_imagelist_new();
+          int i, err;
+          for (i = 0; i < nplanes; i++) {
+            cpl_imagelist_set(maskcube, mask, i);
+          } /* for i (all planes) */
+          /* different mask for the next-to-last plane */
+          cpl_image *mask2 = cpl_image_new(nx, ny, CPL_TYPE_INT);
+          cpl_image_fill_window(mask2, 10, 5, 30, 9, 1);
+          cpl_imagelist_set(maskcube, mask2, nplanes - 2);
+          /* and save the masking cube */
+          cpl_imagelist_save(maskcube, "maskcube_test.fits", CPL_TYPE_INT, NULL,
+                             CPL_IO_CREATE);
+          cpl_imagelist_delete(maskcube);
+          cpl_test_zero(access("maskcube_test.fits", F_OK));
+          /* now do the resampling with bad pixel mask creation */
+          muse_resampling_params *parnocr = muse_resampling_params_new(t);
+          parnocr->tlambda = tl;
+          parnocr->crsigma = -1.;
+          muse_datacube *cb = muse_resampling_cube(pt, parnocr, NULL);
+          muse_resampling_params_delete(parnocr);
+          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 8, 3, &err)));
+          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 12, 5, &err)));
+          cpl_test(!isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 7, 2, &err)));
+          cpl_test(!isnan(cpl_image_get(cpl_imagelist_get(cb->stat, 3), 13, 6, &err)));
+          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, nplanes - 2), 10, 5, &err)));
+          cpl_test(isnan(cpl_image_get(cpl_imagelist_get(cb->stat, nplanes - 2), 30, 9, &err)));
+          muse_datacube_save(cb, "test_cube.fits");
+          muse_datacube_delete(cb);
+          cpl_test_zero(access("badpix_test.fits", F_OK));
+#if MUSE_TEST_RESAMPLING_DELETE_FILES
+          cpl_test(remove("maskcube_test.fits") == 0);
+          cpl_test(remove("test_cube.fits") == 0);
+          cpl_test(remove("badpix_test.fits") == 0);
+#endif
+          cpl_test(unsetenv("MUSE_BADPIX_FROM_MASKCUBE") == 0);
+        }
+
         /* now we are done with both cubes and the parameters */
         prestate = cpl_errorstate_get();
         muse_datacube_delete(cube);
@@ -725,7 +861,7 @@ muse_test_resampling_cube(muse_pixtable *aPtPix)
   cpl_errorstate_set(ps);
   remove("blabla.fits");
 
-  cpl_table_delete(fwhite);
+  muse_table_delete(fwhite);
   return CPL_ERROR_NONE;
 } /* muse_test_resampling_cube() */
 
@@ -948,8 +1084,9 @@ muse_test_resampling_image(muse_pixtable *aPt, muse_pixtable *aPtGeo)
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief   Test program to check that function from muse_resampling and
-           muse_datacube work when called with the necessary data.
+  @brief   Test program to check that function from muse_resampling,
+           muse_datacube and muse_pixgrid work as expected when called with the
+           necessary data.
 
   This program explicitely tests
     muse_resampling_params_new
@@ -971,6 +1108,7 @@ muse_test_resampling_image(muse_pixtable *aPt, muse_pixtable *aPtGeo)
     muse_euro3dcube_save
     muse_datacube_delete
     muse_euro3dcube_delete
+    muse_pixgrid_create
  */
 /*----------------------------------------------------------------------------*/
 int main(int argc, char **argv)
